@@ -3,13 +3,9 @@ package SVN::Web::View;
 use strict;
 use warnings;
 
-use SVN::Core;
-use SVN::Repos;
-use SVN::Fs;
-
 use base 'SVN::Web::action';
 
-our $VERSION = 0.49;
+our $VERSION = 0.50;
 
 =head1 NAME
 
@@ -23,6 +19,10 @@ In F<config.yaml>
     ...
     view:
       class: SVN::Web::View
+      action_menu:
+        show:
+          - file
+        link_text: (view file)
     ...
 
 =head1 DESCRIPTION
@@ -54,6 +54,10 @@ less than C<rev>.
 A boolean value, indicating whether the user is currently viewing the
 HEAD of the file in the repository.
 
+=item context
+
+Always C<file>.
+
 =item rev
 
 The revision that has been returned.  This is not necessarily the same
@@ -101,28 +105,12 @@ sub _log {
 
     return unless $rev > 0;
 
-    my $data = {
+    return {
         rev    => $rev,
         author => $author,
         date   => $self->format_svn_timestamp($date),
         msg    => $msg
     };
-
-    my $root    = $self->{repos}->fs()->revision_root($rev);
-    my $subpool = SVN::Pool->new($pool);
-    $data->{paths} = {
-        map {
-            $_ => {
-                action      => $paths->{$_}->action(),
-                copyfrom    => $paths->{$_}->copyfrom_path(),
-                copyfromrev => $paths->{$_}->copyfrom_rev(),
-                isdir => $root->check_path($_, $subpool) == $SVN::Node::dir,
-                },
-                $subpool->clear()
-            } keys %$paths
-    };
-
-    return $data;
 }
 
 sub cache_key {
@@ -136,36 +124,58 @@ sub cache_key {
 
 sub run {
     my $self = shift;
-    my $pool = SVN::Pool->new_default_sub;
-    my $fs   = $self->{repos}->fs;
+    my $ctx  = $self->{repos}{client};
+    my $ra   = $self->{repos}{ra};
+    my $uri  = $self->{repos}{uri};
     my $path = $self->{path};
 
     my($exp_rev, $yng_rev, $act_rev, $head) = $self->get_revs();
 
     my $rev = $act_rev;
 
-    my $root = $fs->revision_root($rev);
-
     # Get the log for this revision of the file
-    $self->{repos}->get_logs([$path], $rev - 1, $rev, 1, 0,
+    $ra->get_log([$path], $rev, $rev, 1, 1, 1,
         sub { $self->{REV} = $self->_log(@_) });
 
     # Get the text for this revision of the file
-    $root = $fs->revision_root($rev);
-    my $file = $root->file_contents($path);
-    local $/;
+    my($fh, $fc) = (undef, '');
+    open($fh, '>', \$fc);
+    $ctx->cat($fh, $uri . $path, $rev);
+    close($fc);
+
+    my $mime_type;
+    my $props = $ctx->propget('svn:mime-type', $uri . $path, $rev, 0);
+    if(exists $props->{$uri . $path}) {
+	$mime_type = $props->{$uri . $path};
+    } else {
+	$mime_type = 'text/plain';
+    }
+
     return {
         template => 'view',
         data     => {
+	    context      => 'file',
             rev          => $act_rev,
             youngest_rev => $yng_rev,
 	    at_head      => $head,
-            mimetype     => $root->node_prop($path, 'svn:mime-type')
-                || 'text/plain',
-            file => <$file>,
+            mimetype     => $mime_type,
+            file => $fc,
             %{ $self->{REV} },
         }
     };
 }
 
 1;
+
+=head1 COPYRIGHT
+
+Copyright 2003-2004 by Chia-liang Kao C<< <clkao@clkao.org> >>.
+
+Copyright 2005-2007 by Nik Clayton C<< <nik@FreeBSD.org> >>.
+
+This program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
+
+See L<http://www.perl.com/perl/misc/Artistic.html>
+
+=cut
